@@ -29,8 +29,8 @@ extern "C" {
 #endif
 
 #define SUBRANDR_MAJOR 1
-#define SUBRANDR_MINOR 0
-#define SUBRANDR_PATCH 1
+#define SUBRANDR_MINOR 1
+#define SUBRANDR_PATCH 0
 
 typedef int32_t sbr_26dot6;
 typedef uint32_t sbr_bgra8;
@@ -43,6 +43,9 @@ typedef struct sbr_subtitle_context {
   sbr_26dot6 video_width, video_height;
   sbr_26dot6 padding_left, padding_right, padding_top, padding_bottom;
 } sbr_subtitle_context;
+typedef struct sbr_rect2i {
+  int32_t min_x, min_y, max_x, max_y;
+} sbr_rect2i;
 
 // Construct a new `sbr_library *` that is required to use most subrandr APIs.
 //
@@ -108,58 +111,97 @@ int sbr_renderer_render(sbr_renderer *, sbr_subtitle_context const *,
                         uint32_t t, sbr_bgra8 *buffer, uint32_t width,
                         uint32_t height, uint32_t stride);
 
-#ifdef SBR_UNSTABLE
-// Structure representing a single output piece that resulted from
-// fragmented rendering of a subtitle frame.
+// A single output image that resulted from instanced rendering of
+// a subtitle frame.
 //
-// Pieces are output primitives that may not be fully rasterized yet,
+// Note that even though this is called an "image" it may be a
+// different output primitive internally that is not fully rasterized yet
 // but whose bounding box is known and can be used for packing purposes.
 //
 // The size of this struct is not part of the public ABI,
 // new fields may be added in ABI-compatible releases.
-typedef SBR_UNSTABLE struct sbr_output_piece {
-  int32_t x, y;
+typedef struct sbr_output_image {
   uint32_t width, height;
-  struct sbr_output_piece *next;
-} sbr_output_piece;
+  // This field is always NULL when returned by subrandr and isn't
+  // read or modified by the library.
+  // Can be used to associate custom data with images to simplify packing.
+  void *user_data;
+} sbr_output_image;
 
-typedef SBR_UNSTABLE struct sbr_piece_raster_pass sbr_piece_raster_pass;
-
-// Renders a single subtitle frame to output pieces and immediately
-// begins a piece raster pass which it returns a handle to.
+// A single instance that resulted from instanced rendering of
+// a subtitle frame.
 //
-// See `sbr_renderer_render` for details on parameters.
-sbr_piece_raster_pass *sbr_renderer_render_pieces(sbr_renderer *,
-                                                  sbr_subtitle_context const *,
-                                                  uint32_t t) SBR_UNSTABLE;
+// Instances represent possibly scaled parts of output images.
+// `base` refers to the image this instance is an instance of.
+// `dst_x`, `dst_y` specify where in the output this instance
+// should be composited to.
+// `dst_width`, `dst_height` specify the size this instance's
+// source image part needs to be scaled to.
+// `src_off_x`, `src_off_y` specify at what offset in the
+// base image this instance's source rectangle starts.
+// `src_width`, `src_height` specify the size of this
+// instance's source rectangle.
+//
+// To correctly composite an instance you must (conceptually):
+// 1. Cut out the part of the source image covered by the source rectangle.
+// 2. Scale the result to destination dimensions using bilinear interpolation.
+// 3. Blend the result onto the output at the destination position.
+//
+// The size of this struct is not part of the public ABI,
+// new fields may be added in ABI-compatible releases.
+typedef struct sbr_output_instance {
+  struct sbr_output_instance *next;
+  struct sbr_output_image *base;
+  int32_t dst_x, dst_y;
+  uint32_t dst_width, dst_height;
+  uint32_t src_off_x, src_off_y;
+  uint32_t src_width, src_height;
+} sbr_output_instance;
 
-// Returns the first element of the internal list of output pieces
+typedef struct sbr_instanced_raster_pass sbr_instanced_raster_pass;
+
+// Renders a single subtitle frame to output images and immediately
+// begins an instanced raster pass which it returns a handle to.
+//
+// `clip_rect` is a rectangle that the resulting instances will be clipped to.
+// `flags` must be zero.
+//
+// Calling `sbr_instanced_raster_pass_get_instances` on the resulting raster
+// pass will yield a linked list of instances that must be composited onto the
+// output to correctly display the subtitle frame.
+// Instances must be composited in the order they are returned for correct
+// results.
+//
+// See `sbr_renderer_render` for details on other parameters.
+sbr_instanced_raster_pass *
+sbr_renderer_render_instanced(sbr_renderer *, sbr_subtitle_context const *,
+                              uint32_t t, sbr_rect2i clip_rect, uint64_t flags);
+
+// Returns the first element of the internal list of output image instances
 // that are to be drawn during this raster pass.
-sbr_output_piece const *
-sbr_piece_raster_pass_get_pieces(sbr_piece_raster_pass *) SBR_UNSTABLE;
+sbr_output_instance *
+sbr_instanced_raster_pass_get_instances(sbr_instanced_raster_pass *);
 
-// Rasterizes the provided output piece into the provided pixel buffer at
+// Rasterizes this output image into the provided pixel buffer at
 // a specified offset.
 //
-// `off_x` and `off_y` specify the offset at which to draw the piece.
+// `off_x` and `off_y` specify the offset at which to draw the image.
 // These values may be negative or otherwise out of bounds of the
 // output buffer and the result will be appropriately clipped.
 //
 // The pixel buffer is provided in `buffer`, `width`, `height`, and `stride`
-// same as in `sbr_renderer_render`.
-int sbr_piece_raster_pass_draw_piece(sbr_piece_raster_pass *,
-                                     sbr_output_piece const *piece,
-                                     int32_t off_x, int32_t off_y,
-                                     sbr_bgra8 *buffer, uint32_t width,
-                                     uint32_t height,
-                                     uint32_t stride) SBR_UNSTABLE;
+// like in `sbr_renderer_render`.
+int sbr_output_image_rasterize_into(sbr_output_image const *,
+                                    sbr_instanced_raster_pass *, int32_t off_x,
+                                    int32_t off_y, sbr_bgra8 *buffer,
+                                    uint32_t width, uint32_t height,
+                                    uint32_t stride);
 
 // Marks the provided raster pass as finished.
 //
-// Note that calling this function after a raster pass is *mandatory*,
-// currently failing to do so will be met with an assertion failure.
-void sbr_piece_raster_pass_finish(sbr_piece_raster_pass *) SBR_UNSTABLE;
-#endif
+// Must be called before the parent renderer can be used again.
+// Failing to do so will currently result in an assertion failure.
+void sbr_instanced_raster_pass_finish(sbr_instanced_raster_pass *);
 
 void sbr_renderer_destroy(sbr_renderer *);
 
